#!/usr/bin/perl

package LocIP::Whois;

use strict;
use Net::Whois::Proxy;
use LocIP::Utils;
use Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(&localize_by_whois);


# funkcia lokalizuje IP adresu pouzitim udajov ulozenych na Whois serveroch
# vstupne parametre: $host - IP adresa
#                    $dbh  - deskriptor databazoveho pripojenia na databazu so suradnicami 
# navratova hodnota: pole (zoznam) lokalizacnych udajov  ($country, $city, $lat, $lon, $info)
#                    $country - krajina lokalizacie
#                    $city    - mesto lokalizacie
#                    $lat     - latitude (zemepisna sirka)
#                    $lon     - longitude (zemepisna dlzka)
#                    $info    - textovy popis (nazov organizacie)
sub localize_by_whois {
    my ($host, $dbh) = @_;
    my ($country, $city, $lat, $lon, $info); 
    
    my $whois = new Net::Whois::Proxy;
    my $record = $whois->whois($host); # ulozi odpoved z Whois serveru do retazca record
    #print $record;
    
    # urci format odpovede - RIPE / ARIN / LACNIC:
    my $type;
    if ($record =~ /arin whois database/i) {
        $type = "ARIN";
    }
    elsif ($record =~ /whois\.lacnic\.net/i) {
        $type = "LACNIC"
    }
    else {
        $type = "RIPE";
    }
    
    # lokalizacia z odpovede formatu ARIN:
    if ($type eq "ARIN") { 
		my $stateProv;
    	($country) = get_record_value($record, "country");     # urcenie krajiny
		($stateProv) = get_record_value($record, "stateProv"); # urcenie statu v US
    	($city) = get_record_value($record, "city");           # urcenie mesta
		($info) = get_record_value($record, "orgName");        # urcenie nazvu organizacie

		# nacitanie suradnic z databazy:
		my $sel = $dbh->prepare("SELECT city, latitude, longitude FROM geo_city WHERE country LIKE ? AND region LIKE ? AND city LIKE ? LIMIT 1");
		$sel->execute($country, $stateProv, $city);
		while (my @row = $sel->fetchrow_array) {
			($city, $lat, $lon) = @row; # ulozi mesto, zem. sirku a zem. dlzku z databazy do premennych
		}
		# ak sa nepodarili urcit suradnice mesta, urcia sa aspon suradnice statu v USA
		if (! $lon) {
            my $sel = $dbh->prepare("SELECT latitude, longitude FROM geo_city WHERE city = '' AND country LIKE ? AND region LIKE ? LIMIT 1");
    		$sel->execute($country, $stateProv);
    		while (my @row = $sel->fetchrow_array) {
    			($lat, $lon) = @row; # ulozi zem. sirku a zem. dlzku z databazy do premennych
    		}    
        }
    }
    # lokalizacia z odpovedi formatu RIPE a LACNIC:
    else { 
    	my %object = get_whois_objects($record); # nacita objekty Whois odpovede do asoc. pola
    	($country) = get_record_value($object{"inetnum"}, "country"); # urcenie krajiny

    	# zostavi pole retazcov pre vyhladavanie mesta podla priotity jednotlivych objektov: 
        my @searchArr;
        if ($type eq "RIPE") { # cast iba pre objekty RIPE
            @searchArr = get_record_value($object{"inetnum"}, "netname");     
            push @searchArr, reverse(get_record_value($object{"inetnum"}, "descr")); 
			($info) = get_record_value($object{"inetnum"}, "descr"); # urcenie nazvu organizacie
    	}
    	elsif ($type eq "LACNIC") { # cast iba pre objekty LACNIC
            @searchArr = get_record_value($object{"inetnum"}, "owner");    
            push @searchArr, reverse(get_record_value($object{"inetnum"}, "address"));
			($info) = get_record_value($object{"inetnum"}, "owner"); # urcenie nazvu organizacie
    	}
    	# cast rovnaka pre RIPE aj LACNIC
    	push @searchArr, reverse(get_record_value($object{"organisation"}, "address")); 
        push @searchArr, reverse(get_record_value($object{"person"}, "address")); 
        push @searchArr, reverse(get_record_value($object{"role"}, "address")); 
    
		# v jednotlivych riadkoch vytvoreneho pola vyhladava mesta z urcenej krajiny, az kym nenajde zhodu
    	LINES: foreach my $line (@searchArr) {
            #print $line, "\n";
            my $sel = $dbh->prepare("SELECT city, latitude, longitude FROM geo_city WHERE country LIKE ? AND length(city) > 3 AND ? REGEXP city"); 
            $sel->execute($country, $line);
            
            my $count = 0; my $best = 999; 
            while (my @row = $sel->fetchrow_array) {
            	my $pos = index($line, $row[0]); # vyhlada poziciu mesta v riadku
            	if ($pos < $best) {              # ak je pozicia lepsia ako najlepsia doteraz zistena, aktualizuje udaje
                	($city, $lat, $lon) = @row;  # ulozi mesto, zem. sirku a zem. dlzku z databazy do premennych
                	$best = $pos;	
                }
                $count++;
            }
            if ($count > 0) { last LINES; } # ak sa podarilo najst mesto, ukoncenie vonkajsieho foreach cyklu        
        }
    }  
    return ($country, $city, $lat, $lon, $info);
}


# funkcia vrati pole hodnot poloziek zadaneho nazvu z objektu Whois odpovede
# vstupne parametre: $objectData - objekt Whois odpovede (string, viacej riadkov)
#                    $recordName - nazov polozky objektu Whois odpovede(string)
# navratova hodnota: pole (zoznam) hodnot poloziek zadaneho nazvu
sub get_record_value {
    my ($objectData, $recordName) = @_;
    return $objectData =~ /\b$recordName:\s+(.*)\n/ig; # zaciatok slova + nazov polozky + 1 a viac prazdnych znakov + cokolvek az do konca riadku / nerozlisovat male/velke pismena + najst vsetky vyskyty 
}


# funkcia rozparsuje Whois odpoved do asociativneho pola, kde kluce su jednotlive objekty odpovede
# vstupne parametre: $record - odpoved z Whois servera (string, viacej riadkov)
# navratova hodnota: asociativne pole objektov Whois odpovede
sub get_whois_objects {
    my ($record) = @_;
    my (@lines, $line, $varname, $lastvar, $prazdny, %data); 
    
    @lines = split (/\n/, $record); # rozdeli odpoved na riadky
    $prazdny = 1;
	foreach $line (@lines) {
		if ($line =~ /^\s*%/) { next; }
    	if ($line =~ /^\s*(\w*):\s*(.*)$/) { # ziska nazov polozky ($1) a jej hodnotu ($2) -- zaciatok retazca + lubovolny pocet prazdnych znakov + (lubovolny pocet alfanumerickych znakov) + dvojbodka + lubovolny pocet prazdnych znakov + (cokolvek az do konca retazca) 
        	if ($prazdny) { $varname = $1; }
        	$data{$varname} .= trim($1).": ".trim($2)."\n"; # ulozi do asocitivneho pola: "nazovPolozky: hodnotaPolozky"
			$prazdny = 0; $lastvar = $1;
    	}
    	elsif ($line =~ /^\s*$/) { # prazdny riadok
        	$prazdny = 1;
    	}
		elsif ($line =~ /^\s+(.*)$/) { # riadok bez nazvu polozky, ktory je pokracovanim viacriadkovej polozky
			$data{$varname} .= trim($lastvar).": ".trim($1)."\n";
		}
	}
    #print %data;
    return %data;
}
